@@ -12,7 +12,7 @@ import logging
 from . import version, config, option, output, extractor, job, util, exception
 
 __author__ = "Mike Fährmann"
-__copyright__ = "Copyright 2014-2021 Mike Fährmann"
+__copyright__ = "Copyright 2014-2022 Mike Fährmann"
 __license__ = "GPLv2"
 __maintainer__ = "Mike Fährmann"
 __email__ = "mike_faehrmann@web.de"
@@ -22,10 +22,13 @@ __version__ = version.__version__
 def progress(urls, pformat):
     """Wrapper around urls to output a simple progress indicator"""
     if pformat is True:
-        pformat = "[{current}/{total}] {url}"
+        pformat = "[{current}/{total}] {url}\n"
+    else:
+        pformat += "\n"
+
     pinfo = {"total": len(urls)}
     for pinfo["current"], pinfo["url"] in enumerate(urls, 1):
-        print(pformat.format_map(pinfo), file=sys.stderr)
+        output.stderr_write(pformat.format_map(pinfo))
         yield pinfo["url"]
 
 
@@ -35,11 +38,11 @@ def parse_inputfile(file, log):
     Lines starting with '#' and empty lines will be ignored.
     Lines starting with '-' will be interpreted as a key-value pair separated
       by an '='. where 'key' is a dot-separated option name and 'value' is a
-      JSON-parsable value for it. These config options will be applied while
+      JSON-parsable value. These configuration options will be applied while
       processing the next URL.
     Lines starting with '-G' are the same as above, except these options will
-      be valid for all following URLs, i.e. they are Global.
-    Everything else will be used as potential URL.
+      be applied for *all* following URLs, i.e. they are Global.
+    Everything else will be used as a potential URL.
 
     Example input file:
 
@@ -54,7 +57,8 @@ def parse_inputfile(file, log):
     https://example.org/
 
     # next URL uses default filename and 'skip' is false.
-    https://example.com/index.htm
+    https://example.com/index.htm # comment1
+    https://example.com/404.htm   # comment2
     """
     gconf = []
     lconf = []
@@ -91,6 +95,10 @@ def parse_inputfile(file, log):
 
         else:
             # url
+            if " #" in line:
+                line = line.partition(" #")[0].rstrip()
+            elif "\t#" in line:
+                line = line.partition("\t#")[0].rstrip()
             if gconf or lconf:
                 yield util.ExtendedUrl(line, gconf, lconf)
                 gconf = []
@@ -116,9 +124,12 @@ def main():
         if args.yamlfiles:
             config.load(args.yamlfiles, strict=True, fmt="yaml")
         if args.filename:
-            if args.filename == "/O":
-                args.filename = "{filename}.{extension}"
-            config.set((), "filename", args.filename)
+            filename = args.filename
+            if filename == "/O":
+                filename = "{filename}.{extension}"
+            elif filename.startswith("\\f"):
+                filename = "\f" + filename[2:]
+            config.set((), "filename", filename)
         if args.directory:
             config.set((), "base-directory", args.directory)
             config.set((), "directory", ())
@@ -128,6 +139,10 @@ def main():
             config.set((), "skip", "abort:" + str(args.abort))
         if args.terminate:
             config.set((), "skip", "terminate:" + str(args.terminate))
+        if args.cookies_from_browser:
+            browser, _, profile = args.cookies_from_browser.partition(":")
+            browser, _, keyring = browser.partition("+")
+            config.set((), "cookies", (browser, profile, keyring))
         for opts in args.options:
             config.set(*opts)
 
@@ -144,6 +159,21 @@ def main():
                 else:
                     signal.signal(signal_num, signal.SIG_IGN)
 
+        # enable ANSI escape sequences on Windows
+        if util.WINDOWS and config.get(("output",), "ansi"):
+            from ctypes import windll, wintypes, byref
+            kernel32 = windll.kernel32
+            mode = wintypes.DWORD()
+
+            for handle_id in (-11, -12):  # stdout and stderr
+                handle = kernel32.GetStdHandle(handle_id)
+                kernel32.GetConsoleMode(handle, byref(mode))
+                if not mode.value & 0x4:
+                    mode.value |= 0x4
+                    kernel32.SetConsoleMode(handle, mode)
+
+            output.ANSI = True
+
         # extractor modules
         modules = config.get(("extractor",), "modules")
         if modules is not None:
@@ -152,6 +182,12 @@ def main():
             extractor.modules = modules
             extractor._module_iter = iter(modules)
 
+        # format string separator
+        separator = config.get((), "format-separator")
+        if separator:
+            from . import formatter
+            formatter._SEPARATOR = separator
+
         # loglevels
         output.configure_logging(args.loglevel)
         if args.loglevel >= logging.ERROR:
@@ -189,20 +225,23 @@ def main():
                 pass
 
         if args.list_modules:
-            for module_name in extractor.modules:
-                print(module_name)
+            extractor.modules.append("")
+            sys.stdout.write("\n".join(extractor.modules))
+
         elif args.list_extractors:
+            write = sys.stdout.write
+            fmt = "{}\n{}\nCategory: {} - Subcategory: {}{}\n\n".format
+
             for extr in extractor.extractors():
                 if not extr.__doc__:
                     continue
-                print(extr.__name__)
-                print(extr.__doc__)
-                print("Category:", extr.category,
-                      "- Subcategory:", extr.subcategory)
                 test = next(extr._get_tests(), None)
-                if test:
-                    print("Example :", test[0])
-                print()
+                write(fmt(
+                    extr.__name__, extr.__doc__,
+                    extr.category, extr.subcategory,
+                    "\nExample : " + test[0] if test else "",
+                ))
+
         elif args.clear_cache:
             from . import cache
             log = logging.getLogger("cache")
