@@ -6,7 +6,7 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation.
 
-"""Extractors for 4chan archives based on FoolFuuka"""
+"""Extractors for FoolFuuka 4chan archives"""
 
 from .common import BaseExtractor, Message
 from .. import text
@@ -16,6 +16,7 @@ import itertools
 class FoolfuukaExtractor(BaseExtractor):
     """Base extractor for FoolFuuka based boards/archives"""
     basecategory = "foolfuuka"
+    filename_fmt = "{timestamp_ms} {filename_media}.{extension}"
     archive_fmt = "{board[shortname]}_{num}_{timestamp}"
     external = "default"
 
@@ -40,6 +41,9 @@ class FoolfuukaExtractor(BaseExtractor):
 
             post["filename"], _, post["extension"] = \
                 media["media"].rpartition(".")
+            post["filename_media"] = media["media_filename"].rpartition(".")[0]
+            post["timestamp_ms"] = text.parse_int(
+                media["media_orig"].rpartition(".")[0])
             yield Message.Url, url, post
 
     def metadata(self):
@@ -66,6 +70,7 @@ BASE_PATTERN = FoolfuukaExtractor.update({
     },
     "archivedmoe": {
         "root": "https://archived.moe",
+        "pattern": r"archived\.moe",
     },
     "archiveofsins": {
         "root": "https://archiveofsins.com",
@@ -73,16 +78,23 @@ BASE_PATTERN = FoolfuukaExtractor.update({
     },
     "b4k": {
         "root": "https://arch.b4k.co",
+        "pattern": r"arch\.b4k\.co",
     },
     "desuarchive": {
         "root": "https://desuarchive.org",
+        "pattern": r"desuarchive\.org",
     },
     "fireden": {
         "root": "https://boards.fireden.net",
+        "pattern": r"boards\.fireden\.net",
     },
-    "nyafuu": {
-        "root": "https://archive.nyafuu.org",
-        "pattern": r"(?:archive\.)?nyafuu\.org",
+    "rozenarcana": {
+        "root": "https://archive.alice.al",
+        "pattern": r"(?:archive\.)?alice\.al",
+    },
+    "tokyochronos": {
+        "root": "https://www.tokyochronos.net",
+        "pattern": r"(?:www\.)?tokyochronos\.net",
     },
     "rbt": {
         "root": "https://rbt.asia",
@@ -90,9 +102,7 @@ BASE_PATTERN = FoolfuukaExtractor.update({
     },
     "thebarchive": {
         "root": "https://thebarchive.com",
-    },
-    "wakarimasen": {
-        "root": "https://archive.wakarimasen.moe",
+        "pattern": r"thebarchive\.com",
     },
 })
 
@@ -101,11 +111,11 @@ class FoolfuukaThreadExtractor(FoolfuukaExtractor):
     """Base extractor for threads on FoolFuuka based boards/archives"""
     subcategory = "thread"
     directory_fmt = ("{category}", "{board[shortname]}",
-                     "{thread_num}{title:? - //}")
+                     "{thread_num} {title|comment[:50]}")
     pattern = BASE_PATTERN + r"/([^/?#]+)/thread/(\d+)"
     test = (
         ("https://archive.4plebs.org/tg/thread/54059290", {
-            "url": "07452944164b602502b02b24521f8cee5c484d2a",
+            "url": "fd823f17b5001442b941fddcd9ec91bafedfbc79",
         }),
         ("https://archived.moe/gd/thread/309639/", {
             "url": "fdd533840e2d535abd162c02d6dfadbc12e2dcd8",
@@ -127,8 +137,11 @@ class FoolfuukaThreadExtractor(FoolfuukaExtractor):
         ("https://boards.fireden.net/sci/thread/11264294/", {
             "url": "61cab625c95584a12a30049d054931d64f8d20aa",
         }),
-        ("https://archive.nyafuu.org/c/thread/2849220/", {
-            "url": "bbe6f82944a45e359f5c8daf53f565913dc13e4f",
+        ("https://archive.alice.al/c/thread/2849220/", {
+            "url": "632e2c8de05de6b3847685f4bf1b4e5c6c9e0ed5",
+        }),
+        ("https://www.tokyochronos.net/a/thread/241664141/", {
+            "url": "ae03852cf44e3dcfce5be70274cb1828e1dbb7d6",
         }),
         ("https://rbt.asia/g/thread/61487650/", {
             "url": "fadd274b25150a1bdf03a40c58db320fa3b617c4",
@@ -139,9 +152,6 @@ class FoolfuukaThreadExtractor(FoolfuukaExtractor):
         ("https://thebarchive.com/b/thread/739772332/", {
             "url": "e8b18001307d130d67db31740ce57c8561b5d80c",
         }),
-        ("https://archive.wakarimasen.moe/a/thread/223157648/", {
-            "url": "fef0758d2eb81b1ba783051fd5ec491d70107a78",
-        }),
     )
 
     def __init__(self, match):
@@ -177,10 +187,10 @@ class FoolfuukaBoardExtractor(FoolfuukaExtractor):
         ("https://arch.b4k.co/meta/"),
         ("https://desuarchive.org/a/"),
         ("https://boards.fireden.net/sci/"),
-        ("https://archive.nyafuu.org/c/"),
+        ("https://archive.alice.al/c/"),
+        ("https://www.tokyochronos.net/a/"),
         ("https://rbt.asia/g/"),
         ("https://thebarchive.com/b/"),
-        ("https://archive.wakarimasen.moe/a/"),
     )
 
     def __init__(self, match):
@@ -221,10 +231,10 @@ class FoolfuukaSearchExtractor(FoolfuukaExtractor):
         ("https://archiveofsins.com/_/search/text/test/"),
         ("https://desuarchive.org/_/search/text/test/"),
         ("https://boards.fireden.net/_/search/text/test/"),
-        ("https://archive.nyafuu.org/_/search/text/test/"),
+        ("https://archive.alice.al/_/search/text/test/"),
+        ("https://www.tokyochronos.net/_/search/text/test/"),
         ("https://rbt.asia/_/search/text/test/"),
         ("https://thebarchive.com/_/search/text/test/"),
-        ("https://archive.wakarimasen.moe/a/search/text/test/"),
     )
 
     def __init__(self, match):
@@ -287,10 +297,10 @@ class FoolfuukaGalleryExtractor(FoolfuukaExtractor):
         ("https://arch.b4k.co/meta/gallery/"),
         ("https://desuarchive.org/a/gallery/5"),
         ("https://boards.fireden.net/sci/gallery/6"),
-        ("https://archive.nyafuu.org/c/gallery/7"),
+        ("https://archive.alice.al/c/gallery/7"),
+        ("https://www.tokyochronos.net/a/gallery/7"),
         ("https://rbt.asia/g/gallery/8"),
         ("https://thebarchive.com/b/gallery/9"),
-        ("https://archive.wakarimasen.moe/a/gallery/10"),
     )
 
     def __init__(self, match):
