@@ -33,6 +33,7 @@ class KemonopartyExtractor(Extractor):
             self.cookiedomain = ".coomer.party"
         self.root = text.root_from_url(match.group(0))
         Extractor.__init__(self, match)
+        self.session.headers["Referer"] = self.root + "/"
 
     def items(self):
         self._prepare_ddosguard_cookies()
@@ -46,7 +47,7 @@ class KemonopartyExtractor(Extractor):
         comments = self.config("comments")
         username = dms = None
 
-        # prevent files to be sent with gzip compression
+        # prevent files from being sent with gzip compression
         headers = {"Accept-Encoding": "identity"}
 
         if self.config("metadata"):
@@ -63,6 +64,9 @@ class KemonopartyExtractor(Extractor):
 
         for post in posts:
 
+            headers["Referer"] = "{}/{}/user/{}/post/{}".format(
+                self.root, post["service"], post["user"], post["id"])
+            post["_http_headers"] = headers
             post["date"] = text.parse_datetime(
                 post["published"] or post["added"],
                 "%a, %d %b %Y %H:%M:%S %Z")
@@ -74,34 +78,42 @@ class KemonopartyExtractor(Extractor):
                 if dms is True:
                     dms = self._extract_dms(post)
                 post["dms"] = dms
-            yield Message.Directory, post
 
+            files = []
             hashes = set()
-            post["num"] = 0
+
             for file in itertools.chain.from_iterable(
                     g(post) for g in generators):
                 url = file["path"]
 
                 match = find_hash(url)
                 if match:
-                    post["hash"] = hash = match.group(1)
+                    file["hash"] = hash = match.group(1)
                     if hash in hashes and not duplicates:
                         self.log.debug("Skipping %s (duplicate)", url)
                         continue
                     hashes.add(hash)
                 else:
-                    post["hash"] = ""
+                    file["hash"] = ""
 
+                files.append(file)
+
+            post["count"] = len(files)
+            yield Message.Directory, post
+
+            for post["num"], file in enumerate(files, 1):
+                post["hash"] = file["hash"]
                 post["type"] = file["type"]
-                post["num"] += 1
-                post["_http_headers"] = headers
+                url = file["path"]
+
+                text.nameext_from_url(file.get("name", url), post)
+                if not post["extension"]:
+                    post["extension"] = text.ext_from_url(url)
 
                 if url[0] == "/":
                     url = self.root + "/data" + url
                 elif url.startswith(self.root):
                     url = self.root + "/data" + url[20:]
-
-                text.nameext_from_url(file.get("name", url), post)
                 yield Message.Url, url, post
 
     def login(self):
@@ -234,6 +246,7 @@ class KemonopartyPostExtractor(KemonopartyExtractor):
             "keyword": {
                 "added": "Wed, 06 May 2020 20:28:02 GMT",
                 "content": str,
+                "count": 1,
                 "date": "dt:2019-08-11 02:09:04",
                 "edited": None,
                 "embed": dict,
@@ -372,17 +385,21 @@ class KemonopartyDiscordExtractor(KemonopartyExtractor):
             post["channel_name"] = self.channel_name
             post["date"] = text.parse_datetime(
                 post["published"], "%a, %d %b %Y %H:%M:%S %Z")
+            post["count"] = len(files)
             yield Message.Directory, post
 
             for post["num"], file in enumerate(files, 1):
                 post["type"] = file["type"]
                 url = file["path"]
+
+                text.nameext_from_url(file.get("name", url), post)
+                if not post["extension"]:
+                    post["extension"] = text.ext_from_url(url)
+
                 if url[0] == "/":
                     url = self.root + "/data" + url
                 elif url.startswith(self.root):
                     url = self.root + "/data" + url[20:]
-
-                text.nameext_from_url(file["name"], post)
                 yield Message.Url, url, post
 
     def posts(self):
@@ -435,20 +452,44 @@ class KemonopartyDiscordServerExtractor(KemonopartyExtractor):
 class KemonopartyFavoriteExtractor(KemonopartyExtractor):
     """Extractor for kemono.party favorites"""
     subcategory = "favorite"
-    pattern = BASE_PATTERN + r"/favorites"
-    test = ("https://kemono.party/favorites", {
-        "pattern": KemonopartyUserExtractor.pattern,
-        "url": "f4b5b796979bcba824af84206578c79101c7f0e1",
-        "count": 3,
-    })
+    pattern = BASE_PATTERN + r"/favorites(?:/?\?([^#]+))?"
+    test = (
+        ("https://kemono.party/favorites", {
+            "pattern": KemonopartyUserExtractor.pattern,
+            "url": "f4b5b796979bcba824af84206578c79101c7f0e1",
+            "count": 3,
+        }),
+        ("https://kemono.party/favorites?type=post", {
+            "pattern": KemonopartyPostExtractor.pattern,
+            "url": "ecfccf5f0d50b8d14caa7bbdcf071de5c1e5b90f",
+            "count": 3,
+        }),
+    )
+
+    def __init__(self, match):
+        KemonopartyExtractor.__init__(self, match)
+        self.favorites = (text.parse_query(match.group(2)).get("type") or
+                          self.config("favorites") or
+                          "artist")
 
     def items(self):
         self._prepare_ddosguard_cookies()
         self.login()
 
-        users = self.request(self.root + "/api/favorites").json()
-        for user in users:
-            user["_extractor"] = KemonopartyUserExtractor
-            url = "{}/{}/user/{}".format(
-                self.root, user["service"], user["id"])
-            yield Message.Queue, url, user
+        if self.favorites == "artist":
+            users = self.request(
+                self.root + "/api/favorites?type=artist").json()
+            for user in users:
+                user["_extractor"] = KemonopartyUserExtractor
+                url = "{}/{}/user/{}".format(
+                    self.root, user["service"], user["id"])
+                yield Message.Queue, url, user
+
+        elif self.favorites == "post":
+            posts = self.request(
+                self.root + "/api/favorites?type=post").json()
+            for post in posts:
+                post["_extractor"] = KemonopartyPostExtractor
+                url = "{}/{}/user/{}/post/{}".format(
+                    self.root, post["service"], post["user"], post["id"])
+                yield Message.Queue, url, post
