@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2019-2021 Mike Fährmann
+# Copyright 2019-2022 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -193,24 +193,28 @@ class NozomiSearchExtractor(NozomiExtractor):
         return {"search_tags": self.tags}
 
     def posts(self):
-        index = None
-        result = set()
+        result = None
+        positive = []
+        negative = []
 
         def nozomi(path):
             url = "https://j.nozomi.la/" + path + ".nozomi"
             return decode_nozomi(self.request(url).content)
 
         for tag in self.tags:
-            if tag[0] == "-":
-                if not index:
-                    index = set(nozomi("index"))
-                items = index.difference(nozomi("nozomi/" + tag[1:]))
-            else:
-                items = nozomi("nozomi/" + tag)
+            (negative if tag[0] == "-" else positive).append(
+                tag.replace("/", ""))
 
-            if result:
-                result.intersection_update(items)
+        for tag in positive:
+            ids = nozomi("nozomi/" + tag)
+            if result is None:
+                result = set(ids)
             else:
-                result.update(items)
+                result.intersection_update(ids)
 
-        return sorted(result, reverse=True)
+        if result is None:
+            result = set(nozomi("index"))
+        for tag in negative:
+            result.difference_update(nozomi("nozomi/" + tag[1:]))
+
+        return sorted(result, reverse=True) if result else ()
