@@ -16,17 +16,21 @@ class SkebExtractor(Extractor):
     category = "skeb"
     directory_fmt = ("{category}", "{creator[screen_name]}")
     filename_fmt = "{post_num}_{file_id}.{extension}"
-    archive_fmt = "{post_num}_{file_id}_{content_category}"
+    archive_fmt = "{post_num}_{_file_id}_{content_category}"
     root = "https://skeb.jp"
 
     def __init__(self, match):
         Extractor.__init__(self, match)
         self.user_name = match.group(1)
         self.thumbnails = self.config("thumbnails", False)
+        self.article = self.config("article", False)
 
     def items(self):
+        metadata = self.metadata()
         for user_name, post_num in self.posts():
             response, post = self._get_post_data(user_name, post_num)
+            if metadata:
+                post.update(metadata)
             yield Message.Directory, post
             for data in self._get_urls_from_post(response, post):
                 url = data["file_url"]
@@ -35,6 +39,9 @@ class SkebExtractor(Extractor):
     def posts(self):
         """Return post number"""
 
+    def metadata(self):
+        """Return additional metadata"""
+
     def _pagination(self, url, params):
         headers = {"Referer": self.root, "Authorization": "Bearer null"}
         params["offset"] = 0
@@ -64,6 +71,7 @@ class SkebExtractor(Extractor):
         resp = self.request(url, headers=headers).json()
         creator = resp["creator"]
         post = {
+            "post_id"          : resp["id"],
             "post_num"         : post_num,
             "post_url"         : self.root + resp["path"],
             "body"             : resp["body"],
@@ -102,12 +110,22 @@ class SkebExtractor(Extractor):
         if self.thumbnails and "og_image_url" in resp:
             post["content_category"] = "thumb"
             post["file_id"] = "thumb"
+            post["_file_id"] = str(resp["id"]) + "t"
             post["file_url"] = resp["og_image_url"]
             yield post
 
+        if self.article and "article_image_url" in resp:
+            url = resp["article_image_url"]
+            if url:
+                post["content_category"] = "article"
+                post["file_id"] = "article"
+                post["_file_id"] = str(resp["id"]) + "a"
+                post["file_url"] = url
+                yield post
+
         for preview in resp["previews"]:
             post["content_category"] = "preview"
-            post["file_id"] = preview["id"]
+            post["file_id"] = post["_file_id"] = preview["id"]
             post["file_url"] = preview["url"]
             info = preview["information"]
             post["original"] = {
@@ -134,11 +152,12 @@ class SkebPostExtractor(SkebExtractor):
             "anonymous": False,
             "body": "re:はじめまして。私はYouTubeにてVTuberとして活動をしている湊ラ",
             "client": {
-                "avatar_url": "https://pbs.twimg.com/profile_images"
-                              "/1471184042791895042/f0DcWFGl.jpg",
-                "header_url": None,
+                "avatar_url": r"re:https://pbs.twimg.com/profile_images"
+                              r"/\d+/\w+\.jpg",
+                "header_url": r"re:https://pbs.twimg.com/profile_banners"
+                              r"/1375007870291300358/\d+/1500x500",
                 "id": 1196514,
-                "name": "湊ラギ",
+                "name": str,
                 "screen_name": "minato_ragi",
             },
             "completed_at": "2022-02-27T14:03:45.442Z",
@@ -208,3 +227,86 @@ class SkebUserExtractor(SkebExtractor):
             posts = itertools.chain(posts, self._pagination(url, params))
 
         return posts
+
+
+class SkebSearchExtractor(SkebExtractor):
+    """Extractor for skeb search results"""
+    subcategory = "search"
+    pattern = r"(?:https?://)?skeb\.jp/search\?q=([^&#]+)"
+    test = ("https://skeb.jp/search?q=bunny%20tree&t=works", {
+        "count": ">= 18",
+        "keyword": {"search_tags": "bunny tree"},
+    })
+
+    def metadata(self):
+        return {"search_tags": text.unquote(self.user_name)}
+
+    def posts(self):
+        url = "https://hb1jt3kre9-2.algolianet.com/1/indexes/*/queries"
+        params = {
+            "x-algolia-agent": "Algolia for JavaScript (4.13.1); Browser",
+        }
+        headers = {
+            "Origin": self.root,
+            "Referer": self.root + "/",
+            "x-algolia-api-key": "9a4ce7d609e71bf29e977925e4c6740c",
+            "x-algolia-application-id": "HB1JT3KRE9",
+        }
+
+        filters = self.config("filters")
+        if filters is None:
+            filters = ("genre:art OR genre:voice OR genre:novel OR "
+                       "genre:video OR genre:music OR genre:correction")
+        elif not isinstance(filters, str):
+            filters = " OR ".join(filters)
+
+        page = 0
+        pams = "hitsPerPage=40&filters=" + text.quote(filters) + "&page="
+
+        request = {
+            "indexName": "Request",
+            "query": text.unquote(self.user_name),
+            "params": pams + str(page),
+        }
+        data = {"requests": (request,)}
+
+        while True:
+            result = self.request(
+                url, method="POST", params=params, headers=headers, json=data,
+            ).json()["results"][0]
+
+            for post in result["hits"]:
+                parts = post["path"].split("/")
+                yield parts[1][1:], parts[3]
+
+            if page >= result["nbPages"]:
+                return
+            page += 1
+            request["params"] = pams + str(page)
+
+
+class SkebFollowingExtractor(SkebExtractor):
+    """Extractor for all creators followed by a skeb user"""
+    subcategory = "following"
+    pattern = r"(?:https?://)?skeb\.jp/@([^/?#]+)/following_creators"
+    test = ("https://skeb.jp/@user/following_creators",)
+
+    def items(self):
+        for user in self.users():
+            url = "{}/@{}".format(self.root, user["screen_name"])
+            user["_extractor"] = SkebUserExtractor
+            yield Message.Queue, url, user
+
+    def users(self):
+        url = "{}/api/users/{}/following_creators".format(
+            self.root, self.user_name)
+        params = {"sort": "date", "offset": 0, "limit": 90}
+        headers = {"Referer": self.root, "Authorization": "Bearer null"}
+
+        while True:
+            data = self.request(url, params=params, headers=headers).json()
+            yield from data
+
+            if len(data) < params["limit"]:
+                return
+            params["offset"] += params["limit"]
