@@ -30,12 +30,16 @@ class UnsplashExtractor(Extractor):
 
     def items(self):
         fmt = self.config("format") or "raw"
+        metadata = self.metadata()
+
         for photo in self.photos():
             util.delete_items(
                 photo, ("current_user_collections", "related_collections"))
             url = photo["urls"][fmt]
             text.nameext_from_url(url, photo)
 
+            if metadata:
+                photo.update(metadata)
             photo["extension"] = "jpg"
             photo["date"] = text.parse_datetime(photo["created_at"])
             if "tags" in photo:
@@ -44,6 +48,10 @@ class UnsplashExtractor(Extractor):
             yield Message.Directory, photo
             yield Message.Url, url, photo
 
+    @staticmethod
+    def metadata():
+        return None
+
     def skip(self, num):
         pages = num // self.per_page
         self.page_start += pages
@@ -76,7 +84,7 @@ class UnsplashImageExtractor(UnsplashExtractor):
             "blur_hash": "LZP4uQS4jboe%#o0WCa}2doJNaaz",
             "categories": list,
             "color": "#f3c08c",
-            "created_at": "2020-04-08T08:29:42-04:00",
+            "created_at": "2020-04-08T12:29:42Z",
             "date": "dt:2020-04-08 12:29:42",
             "description": "The Island",
             "downloads": int,
@@ -104,7 +112,7 @@ class UnsplashImageExtractor(UnsplashExtractor):
                 },
                 "title": "Beaver Dam, WI 53916, USA"
             },
-            "promoted_at": "2020-04-08T11:12:03-04:00",
+            "promoted_at": "2020-04-08T15:12:03Z",
             "sponsorship": None,
             "tags": list,
             "updated_at": str,
@@ -115,9 +123,9 @@ class UnsplashImageExtractor(UnsplashExtractor):
                 "id": "uMJXuywXLiU",
                 "instagram_username": "just_midwest_rock",
                 "last_name": "Hoefler",
-                "location": "Madison, WI",
+                "location": None,
                 "name": "Dave Hoefler",
-                "portfolio_url": str,
+                "portfolio_url": None,
                 "total_collections": int,
                 "total_likes": int,
                 "total_photos": int,
@@ -172,17 +180,27 @@ class UnsplashFavoriteExtractor(UnsplashExtractor):
 class UnsplashCollectionExtractor(UnsplashExtractor):
     """Extractor for an unsplash collection"""
     subcategory = "collection"
-    pattern = BASE_PATTERN + r"/collections/([^/?#]+)"
+    pattern = BASE_PATTERN + r"/collections/([^/?#]+)(?:/([^/?#]+))?"
     test = (
         ("https://unsplash.com/collections/3178572/winter", {
             "pattern": r"https://images\.unsplash\.com/(photo-\d+-\w+"
                        r"|reserve/[^/?#]+)\?ixid=\w+&ixlib=rb-1\.2\.1$",
+            "keyword": {"collection_id": "3178572",
+                        "collection_title": "winter"},
             "range": "1-30",
             "count": 30,
         }),
+        ("https://unsplash.com/collections/3178572/"),
         ("https://unsplash.com/collections/_8qJQ2bCMWE/2021.05"),
     )
 
+    def __init__(self, match):
+        UnsplashExtractor.__init__(self, match)
+        self.title = match.group(2) or ""
+
+    def metadata(self):
+        return {"collection_id": self.item, "collection_title": self.title}
+
     def photos(self):
         url = "{}/napi/collections/{}/photos".format(self.root, self.item)
         params = {"order_by": "latest"}
