@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
-# Copyright 2019-2021 Mike Fährmann
+# Copyright 2019-2022 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -16,10 +16,10 @@ import logging
 import zipfile
 import tempfile
 import collections
-from datetime import datetime, timezone as tz
+from datetime import datetime
 
 sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
-from gallery_dl import extractor, output, util  # noqa E402
+from gallery_dl import extractor, output, path  # noqa E402
 from gallery_dl import postprocessor, config  # noqa E402
 from gallery_dl.postprocessor.common import PostProcessor  # noqa E402
 
@@ -32,7 +32,7 @@ class FakeJob():
 
     def __init__(self, extr=extractor.find("test:")):
         self.extractor = extr
-        self.pathfmt = util.PathFormat(extr)
+        self.pathfmt = path.PathFormat(extr)
         self.out = output.NullOutput()
         self.get_logger = logging.getLogger
         self.hooks = collections.defaultdict(list)
@@ -329,23 +329,75 @@ class MetadataTest(BasePostprocessorTest):
         path = self.pathfmt.realdirectory + "test_file__meta_.data"
         m.assert_called_once_with(path, "w", encoding="utf-8")
 
+    def test_metadata_stdout(self):
+        self._create({"filename": "-", "indent": None})
+
+        with patch("sys.stdout", Mock()) as m:
+            self._trigger()
+
+        self.assertEqual(self._output(m), """\
+{"category": "test", "extension": "ext", "filename": "file"}
+""")
+
+    def test_metadata_modify(self):
+        kwdict = {"foo": 0, "bar": {"bax": 1, "bay": 2, "baz": 3}}
+        self._create({
+            "mode": "modify",
+            "fields": {
+                "foo"     : "{filename}-{foo!s}",
+                "foo2"    : "\fE bar['bax'] + 122",
+                "bar[baz]": "{_now}",
+                "bar[ba2]": "test",
+            },
+        }, kwdict)
+        pdict = self.pathfmt.kwdict
+
+        self.assertIsNot(kwdict, pdict)
+        self.assertEqual(pdict["foo"], kwdict["foo"])
+        self.assertEqual(pdict["bar"], kwdict["bar"])
+
+        self._trigger()
+
+        self.assertEqual(pdict["foo"]       , "file-0")
+        self.assertEqual(pdict["foo2"]      , 123)
+        self.assertEqual(pdict["bar"]["ba2"], "test")
+        self.assertIsInstance(pdict["bar"]["baz"], datetime)
+
+    def test_metadata_delete(self):
+        kwdict = {"foo": 0, "bar": {"bax": 1, "bay": 2, "baz": 3}}
+        self._create({"mode": "delete", "fields": ["foo", "bar[baz]"]}, kwdict)
+        pdict = self.pathfmt.kwdict
+
+        self.assertIsNot(kwdict, pdict)
+        self.assertEqual(pdict["foo"], kwdict["foo"])
+        self.assertEqual(pdict["bar"], kwdict["bar"])
+
+        del kwdict["foo"]
+        del kwdict["bar"]["baz"]
+
+        self._trigger()
+        self.assertNotIn("foo", pdict)
+        self.assertNotIn("baz", pdict["bar"])
+        self.assertEqual(kwdict["bar"], pdict["bar"])
+
+        self._trigger()
+        self.assertNotIn("foo", pdict)
+        self.assertNotIn("baz", pdict["bar"])
+        self.assertEqual(kwdict["bar"], pdict["bar"])
+
     @staticmethod
     def _output(mock):
         return "".join(
             call[1][0]
             for call in mock.mock_calls
-            if call[0] == "().write"
+            if call[0].endswith("write")
         )
 
 
 class MtimeTest(BasePostprocessorTest):
 
-    def test_mtime_default(self):
-        pp = self._create()
-        self.assertEqual(pp.key, "date")
-
     def test_mtime_datetime(self):
-        self._create(None, {"date": datetime(1980, 1, 1, tzinfo=tz.utc)})
+        self._create(None, {"date": datetime(1980, 1, 1)})
         self._trigger()
         self.assertEqual(self.pathfmt.kwdict["_mtime"], 315532800)
 
@@ -354,31 +406,36 @@ class MtimeTest(BasePostprocessorTest):
         self._trigger()
         self.assertEqual(self.pathfmt.kwdict["_mtime"], 315532800)
 
-    def test_mtime_custom(self):
+    def test_mtime_key(self):
         self._create({"key": "foo"}, {"foo": 315532800})
         self._trigger()
         self.assertEqual(self.pathfmt.kwdict["_mtime"], 315532800)
 
+    def test_mtime_value(self):
+        self._create({"value": "{foo}"}, {"foo": 315532800})
+        self._trigger()
+        self.assertEqual(self.pathfmt.kwdict["_mtime"], 315532800)
+
 
 class ZipTest(BasePostprocessorTest):
 
     def test_zip_default(self):
         pp = self._create()
-        self.assertEqual(self.job.hooks["file"][0], pp.write)
-        self.assertEqual(pp.path, self.pathfmt.realdirectory)
+        self.assertEqual(self.job.hooks["file"][0], pp.write_fast)
+        self.assertEqual(pp.path, self.pathfmt.realdirectory[:-1])
         self.assertEqual(pp.delete, True)
         self.assertEqual(pp.args, (
-            pp.path[:-1] + ".zip", "a", zipfile.ZIP_STORED, True,
+            pp.path + ".zip", "a", zipfile.ZIP_STORED, True,
         ))
         self.assertTrue(pp.args[0].endswith("/test.zip"))
 
     def test_zip_safe(self):
         pp = self._create({"mode": "safe"})
         self.assertEqual(self.job.hooks["file"][0], pp.write_safe)
-        self.assertEqual(pp.path, self.pathfmt.realdirectory)
+        self.assertEqual(pp.path, self.pathfmt.realdirectory[:-1])
         self.assertEqual(pp.delete, True)
         self.assertEqual(pp.args, (
-            pp.path[:-1] + ".zip", "a", zipfile.ZIP_STORED, True,
+            pp.path + ".zip", "a", zipfile.ZIP_STORED, True,
         ))
         self.assertTrue(pp.args[0].endswith("/test.zip"))
 
@@ -390,14 +447,16 @@ class ZipTest(BasePostprocessorTest):
         })
         self.assertEqual(pp.delete, False)
         self.assertEqual(pp.args, (
-            pp.path[:-1] + ".cbz", "a", zipfile.ZIP_DEFLATED, True,
+            pp.path + ".cbz", "a", zipfile.ZIP_DEFLATED, True,
         ))
         self.assertTrue(pp.args[0].endswith("/test.cbz"))
 
     def test_zip_write(self):
-        pp = self._create()
-
         with tempfile.NamedTemporaryFile("w", dir=self.dir.name) as file:
+            pp = self._create({"files": [file.name, "_info_.json"],
+                               "keep-files": True})
+
+            filename = os.path.basename(file.name)
             file.write("foobar\n")
 
             # write dummy file with 3 different names
@@ -409,18 +468,19 @@ class ZipTest(BasePostprocessorTest):
                 self._trigger()
 
                 nti = pp.zfile.NameToInfo
-                self.assertEqual(len(nti), i+1)
+                self.assertEqual(len(nti), i+2)
                 self.assertIn(name, nti)
 
             # check file contents
-            self.assertEqual(len(nti), 3)
+            self.assertEqual(len(nti), 4)
             self.assertIn("file0.ext", nti)
             self.assertIn("file1.ext", nti)
             self.assertIn("file2.ext", nti)
+            self.assertIn(filename, nti)
 
             # write the last file a second time (will be skipped)
             self._trigger()
-            self.assertEqual(len(pp.zfile.NameToInfo), 3)
+            self.assertEqual(len(pp.zfile.NameToInfo), 4)
 
         # close file
         self._trigger(("finalize",), 0)
@@ -428,10 +488,11 @@ class ZipTest(BasePostprocessorTest):
         # reopen to check persistence
         with zipfile.ZipFile(pp.zfile.filename) as file:
             nti = file.NameToInfo
-            self.assertEqual(len(pp.zfile.NameToInfo), 3)
-            self.assertIn("file0.ext", pp.zfile.NameToInfo)
-            self.assertIn("file1.ext", pp.zfile.NameToInfo)
-            self.assertIn("file2.ext", pp.zfile.NameToInfo)
+            self.assertEqual(len(pp.zfile.NameToInfo), 4)
+            self.assertIn("file0.ext", nti)
+            self.assertIn("file1.ext", nti)
+            self.assertIn("file2.ext", nti)
+            self.assertIn(filename, nti)
 
         os.unlink(pp.zfile.filename)
 
